/**********************************************************************

	FINDFIRSTSECTOR.C
	Z-World, 2002

	This program attempts to find the first LBA sector of the first
	file on the first partition of a PCMCIA (or CompactFlash) flash disk.
	This information can be used to let a Rabbit write to a flash disk
	formatted with a PC-compatible file system without having to support
	the actual filesystem in the Rabbit code.

	You will need to provide a string that appears at the start of
	the file to ensure that the correct sector is found.
		
	See Tech Note TN222 for more details.

 **********************************************************************/

#use "atainterface.lib"

/**********************************************************************/

void setupRabbit()
{
	// enable the alternate I/O bus for read/writes with IOE prefix
	enableIObus();

	// initialize port E pins
	WrPortI(PEDR, &PEDRShadow, 0x09);	// set PE0 and PE3 high
	WrPortI(PEDDR, &PEDDRShadow, 0x09);	// set PE0 and PE3 as outputs
	WrPortI(PEFR, &PEFRShadow, 0x01);	// enable I/O strobe on PE0
	WrPortI(IB0CR, &IB0CRShadow, 0xC8);	// PE0 is a chip select strobe
}

//////////////////////////////////////////////////////////////////////

// These macros convert multi-byte values on the disk to the
// proper arrangement for the Rabbit.

#define	READINT(a)		(buf[a] + buf[a+1]*0x100)
#define	READLONG(a)		(buf[a] + \
								 buf[a+1]*0x00000100ul + \
								 buf[a+2]*0x00010000ul + \
								 buf[a+3]*0x01000000ul)

// the string that the dummy file starts with
								 
#define	MATCH_STR		"ABCDEFGHIJKLMNOP"


main()
{
	char	buf[512], oemName[9], fsType[9];
	unsigned long	partitionSector, fileSector, numSectors;
	int	reservedSize, fatSize, numFats;
	int	maxDirEntries, bytesPerSector;

	char	s[32];
	int	i, j, done;
		

	setupRabbit();
	ataInit();

	// read the partition table

	ataReadSector(0, buf);	
	if ((buf[0x1FE] != 0x55) || (buf[0x1FF] != 0xAA)) {
		printf("Master boot record invalid, exiting.\n");
		exit(1);
	}		
	partitionSector = READLONG(0x1C6);
	printf("The first partition starts at sector %ld\n",
				partitionSector);
	

	// now read the partition boot record

	ataReadSector(partitionSector, buf);
	if ((buf[0x1FE] != 0x55) || (buf[0x1FF] != 0xAA)) {
		printf("Partition boot record invalid, exiting.\n");
		exit(1);
	}
	memcpy(oemName, &buf[0x03], 8);	oemName[8] = '\0';
	memcpy(fsType,  &buf[0x36], 8);	fsType[8]  = '\0';
	printf("This file system is type %s\n", fsType);
	printf("It claims to be created by %s\n\n", oemName);

	numSectors = READINT(0x13);
	if (numSectors == 0x00)		numSectors = READLONG(0x20);
			
	reservedSize = READINT(0x0E);
	bytesPerSector = READINT(0x0B);
	numFats = buf[0x10];
	maxDirEntries = READINT(0x11);
	fatSize = READINT(0x16);


	printf("There are %ld sectors of %d bytes each on this partition.\n",
				numSectors, bytesPerSector);
	printf("The FAT table(s) start at sector %ld\n",
				partitionSector + reservedSize);
	printf("The root directory starts at sector %ld\n",
				partitionSector + reservedSize +
				numFats*fatSize);
				
	fileSector = partitionSector +
					 reservedSize +
					 numFats*fatSize +
					 (maxDirEntries*32)/bytesPerSector;


	// now do the last search for the start of the first file
					 
	memset(buf, 0x00, 512);

	ataReadSector(fileSector, buf);
	buf[strlen(MATCH_STR)] = '\0';
	while (strncmp(buf, MATCH_STR, strlen(MATCH_STR))) {
		fileSector++;
		ataReadSector(fileSector, buf);
		buf[strlen(MATCH_STR)] = '\0';
	}

	printf("\nFirst matching sector at:  %d\n", fileSector);
}
